#include <QtGui>

#include "MultiDelegate.h"

MultiDelegate::MultiDelegate(QWidget *parent) 
	: QStyledItemDelegate(parent) 
{
	m_table=qobject_cast<QTableWidget *>(parent);
}

QWidget *MultiDelegate::createEditor(QWidget *parent,
                                    const QStyleOptionViewItem & /*option*/,
                                    const QModelIndex &/*index*/) const

{
	QPlainTextEdit *editor = new QPlainTextEdit(parent);
	return editor;
}

void MultiDelegate::setEditorData(QWidget *editor,
                                 const QModelIndex &index) const
{
	QPlainTextEdit *mEditor = qobject_cast<QPlainTextEdit *>(editor);
	mEditor->setPlainText(index.data().toString());
}

void MultiDelegate::setModelData(QWidget *editor, QAbstractItemModel *model,
                                const QModelIndex &index) const
{
	QPlainTextEdit *mEditor = qobject_cast<QPlainTextEdit *>(editor);
	model->setData(index, mEditor->toPlainText());
}

bool MultiDelegate::eventFilter ( QObject * editor, QEvent * oevent )
{
	QPlainTextEdit *mEditor = qobject_cast<QPlainTextEdit *>(editor);
	if (!mEditor || !oevent)
		return false;


	int nevent_type=oevent->type();
	if (
		nevent_type== QEvent::KeyPress ||
		nevent_type== QEvent::KeyRelease
		) {
		QKeyEvent *keyevt=static_cast<QKeyEvent *>(oevent);
		if(keyevt){
			switch(keyevt->key()){
			case Qt::Key_Enter:
			case Qt::Key_Return:
				if(nevent_type==QEvent::KeyRelease){
					return true;
				}
				if(keyevt->modifiers()==Qt::ControlModifier){
					mEditor->insertPlainText("\n");
					return true;
				} else {
					emit commitData(mEditor);
					emit closeEditor(mEditor);
					if(m_table){
						int ncol=m_table->currentColumn();
						int nrow=m_table->currentRow();
						nrow++;
						if(nrow>=m_table->rowCount()){
							nrow=0;
						}
						m_table->setCurrentCell(nrow,ncol);

					}
					return true;
				}
				return true;
				break;
			default:
				break;
			}
		}
	}
	return QStyledItemDelegate::eventFilter( editor,oevent );
}
